// -*- C++ -*-
#ifndef FHERWIG_HH 
#define FHERWIG_HH

#include "AGILe/Generator.hh"

namespace AGILe {

  /// Abstract base class for the Fortran-based Herwig generator series
  /// @author Andy Buckley
  /// @date   2006-06-12
  class FHerwig : public Generator {
  public:
    /// Standard constructor
    FHerwig();

    /// @todo Can we avoid the virtual function overhead? Is it really
    /// significant, considering it's added to all the underlying generator
    /// computation and complexity? Design considerations, polymorphism: should
    /// *all* generators make all methods virtual "just in case" they're
    /// eventually subclassed? (Ans: probably not, better to make them virtual
    /// "on demand" when a situation like the Jimmy subclassing happens.)
    
    /// Destructor
    virtual ~FHerwig() {}

    /// Set up initial state from supplied params.
    virtual Status setGenSpecificInitialState(PdgCode p1 = PROTON, double e1 = 7000.0, 
                                   PdgCode p2 = PROTON, double e2 = 7000.0);
    
    /// Run the generator for one event.
    virtual Status makeEvent(HepMC::GenEvent& evt);

    /// Set the random number generator seed.
    virtual Status setSeed(const int value);

    /// Make sure that generic setParam is visible.
    using Generator::setParam;
    
    /// Pass a parameter to the generator.
    virtual Status setParam(const string& name, const string& value);
    
    /// Tidy up after ourselves
    virtual Status finalize();
    
    virtual const string getName() const{
      return "HERWIG";
    };
    
    string getPDFSet(PdgCode pid);
    int getPDFMember(PdgCode pid);
    string getPDFScheme(PdgCode pid)const;

    const virtual double getCrossSection();
    
  protected:
    /// Set up default params etc.
    virtual Status initialize();

    /// Fill a HepMC event
    virtual Status fillEvent(HepMC::GenEvent& evt);

  private:
    
    int beamNumber(PdgCode pid);
    enum {AUTPDF_LENGTH=20};
    enum {SEED_OFFSET=17};

  };
  
}

#endif
  
